#' Table of Significant Subpathways
#'
#' Create a table of all significant subpathways
#'
#' @param path_results Results data frame generated by
#'  \code{\link{subpathway_analysis}}
#'
#' @return A table of all significant subpathways. Including the significant
#'  model type
#' and model type p-value.
#'
#' @examples
#' # Load data
#' data("demoDataSmall", package = "MetabolomicsPipeline")
#' dat <- demoDataSmall
#'
#' # Runsubpathay analysis
#' sub_analysis <- subpathway_analysis(dat,
#'     treat_var = "GROUP_NAME",
#'     block_var = "TIME1",
#'     strat_var = NULL,
#'     Assay = "normalized"
#' )
#'
#' #############################################################################
#' ### Results Plots ###########################################################
#' #############################################################################
#'
#' # significant subpathways by model type
#' subpath_by_model(sub_analysis)
#'
#' # Percentage of signficant subpathways within superpathways
#' subpath_within_superpath(sub_analysis)
#'
#' met_within_sub(sub_analysis, subpathway = "Aminosugar Metabolism")
#'
#' # All signifiicant subpathways
#' all_sig_subpath(sub_analysis)
#'
#' @importFrom  dplyr case_when
#' @importFrom  dplyr mutate
#' @importFrom  dplyr select
#' @importFrom dplyr distinct
#' @importFrom dplyr filter
#' @importFrom dplyr arrange
#' @importFrom knitr kable
#' @importFrom kableExtra kable_classic
#' @export


all_sig_subpath <- function(path_results) {
    # Structure levels
    if (sum(grepl("interaction", names(path_results))) == 0) {
        levs <- c("Single", "None")

        cases <- expression(dplyr::case_when(model == "Single" ~ single_fisher))
    }
    if (!sum(grepl("interaction", names(path_results))) == 0) {
        levs <- c("Interaction", "Parallel", "Single", "None")

        cases <- expression(dplyr::case_when(
            model == "Interaction" ~ interaction_fisher,
            model == "Parallel" ~ parallel_fisher,
            model == "Single" ~ single_fisher
        ))
    }



    # Create pathway table
    pathway_table <- path_results %>% #
        dplyr::mutate(model = factor(model, levels = levs)) %>%
        dplyr::select(sub_pathway, ends_with("_fisher"), model) %>%
        dplyr::distinct() %>%
        dplyr::filter(model != "None") %>%
        dplyr::mutate(pval = round(eval(cases), 3)) %>%
        dplyr::select(sub_pathway, model, pval) %>%
        dplyr::arrange(model, pval)


    # Format table
    path_tab <- pathway_table %>%
        knitr::kable(
            col.names = c("Subpathway", "Model Type", "P-value"),
            caption = "Significant Subpathways"
        ) %>%
        kableExtra::kable_classic(full_width = FALSE, html_font = "Cambria")


    # return table
    return(path_tab)
}
