#' Metabolites within Subpathway Table
#'
#' Return the model results for each metabolite within a subpathway.
#'
#' @param subpath_results  Results data frame generated by
#'  \code{\link{subpathway_analysis}}
#'
#' @param subpathway Character string of the subpathway of interest.
#' This is case sensitive and must be listed in the subpath_results.
#'
#' @param mod Model of interest. This can be a single model or a vector of
#'  model types that can take on the values "interaction", "parallel",
#'  or "single".
#'
#' @returns A table with the results from the model types specified and for each
#' metabolite within the superpathway specified.
#'
#' @importFrom dplyr filter
#' @importFrom dplyr select
#' @importFrom knitr kable
#' @importFrom kableExtra kable_paper
#' @importFrom stringr str_to_title
#'
#' @examples
#' data("demoDataSmall", package = "MetabolomicsPipeline")
#' dat <- demoDataSmall
#'
#' # Runsubpathay analysis
#' sub_analysis <- subpathway_analysis(dat,
#'     treat_var = "GROUP_NAME",
#'     block_var = "TIME1",
#'     strat_var = NULL,
#'     Assay = "normalized"
#' )
#'
#' #############################################################################
#' ### Results Plots ###########################################################
#' #############################################################################
#'
#' # significant subpathways by model type
#' subpath_by_model(sub_analysis)
#'
#' # Percentage of signficant subpathways within superpathways
#' subpath_within_superpath(sub_analysis)
#'
#' met_within_sub(sub_analysis, subpathway = "Aminosugar Metabolism")
#'
#' @export
#'
#'


met_within_sub <- function(subpath_results, subpathway,
                        mod = c("interaction", "parallel", "single")) {
    if (inherits(subpath_results, "data.frame")) {
        # Get model results in dataframe
        mod_res <-
            paste0(mod, "_pval")[
                paste0(mod, "_pval") %in% names(subpath_results)
            ]

        # create table
        table <- subpath_results %>%
            dplyr::filter(sub_pathway == subpathway) %>%
            dplyr::select(chem_name, all_of(mod_res)) %>%
            knitr::kable(
                digits = 3,
                col.names = c(
                    "Metabolite Name",
                    stringr::str_to_title(paste0(mod_res, " P-Value"))
                ),
                caption = paste0(
                    "Metabolites within ",
                    stringr::str_to_title(subpathway)
                )
            ) %>%
            kableExtra::kable_paper(full_width = FALSE, html_font = "Cambria")

        return(table)
    }

    if (inherits(subpath_results, "list")) {
        tables <- lapply(names(subpath_results), function(name) {
            stratum <- subpath_results[[name]]

            # Get model results in dataframe
            mod_res <- paste0(mod, "_pval")[
                paste0(mod, "_pval") %in% names(stratum)
            ]

            stratum %>%
                dplyr::filter(sub_pathway == subpathway) %>%
                dplyr::select(chem_name, all_of(mod_res)) %>%
                knitr::kable(
                    digits = 3,
                    col.names = c(
                        "Metabolite Name",
                        stringr::str_to_title(paste0(mod_res, " P-Value"))
                    ),
                    caption = paste0(
                        "Metabolites within ",
                        str_to_title(subpathway),
                        " (", name, ")"
                    )
                ) %>%
                kableExtra::kable_paper(
                    full_width = FALSE,
                    html_font = "Cambria"
                )
        })

        return(tables)
    }
}
