#' Subpathway model type table
#'
#' Create a table with the number of significant subpathways for each model
#'  type.
#'
#' @param subpath_results Results data frame generated by
#'  \code{\link{subpathway_analysis}}
#'
#' @returns A table of the number of significant subpathways by model type.
#'
#' @details
#' Each subpathway will only have one model type. We first test the interaction,
#'  and then the parallel and single models are tested last. Suppose a
#'  subpathway has a significant interaction model type. In that case, the table
#'   will count it as an interaction and not as a parallel or single.
#'
#' @examples
#' # Load data
#' data("demoDataSmall", package = "MetabolomicsPipeline")
#' dat <- demoDataSmall
#'
#' # Runsubpathay analysis
#' sub_analysis <- subpathway_analysis(dat,
#'     treat_var = "GROUP_NAME",
#'     block_var = "TIME1",
#'     strat_var = NULL,
#'     Assay = "normalized"
#' )
#'
#' #############################################################################
#' ### Results Plots ###########################################################
#' #############################################################################
#'
#' # significant subpathways by model type
#' subpath_by_model(sub_analysis)
#'
#' # Percentage of signficant subpathways within superpathways
#' subpath_within_superpath(sub_analysis)
#'
#' met_within_sub(sub_analysis, subpathway = "Aminosugar Metabolism")
#'
#' @importFrom dplyr mutate
#' @importFrom dplyr select
#' @importFrom dplyr distinct
#' @importFrom dplyr count
#' @importFrom knitr kable
#' @importFrom dplyr left_join
#' @importFrom kableExtra kable_paper
#'
#' @export
#'

subpath_by_model <- function(subpath_results) {
    if (inherits(subpath_results, "data.frame")) {
        # 2. Structure levels
        if (sum(grepl("interaction", names(subpath_results))) == 0) {
            levs <- c("Single", "None")
        }
        if (!sum(grepl("interaction", names(subpath_results))) == 0) {
            levs <- c("Interaction", "Parallel", "Single", "None")
        }


        # 3. Create table data
        table_data <- subpath_results %>%
            dplyr::mutate(model = factor(model, levels = levs)) %>%
            dplyr::select(sub_pathway, ends_with("_fisher"), model) %>%
            dplyr::distinct()


        # 4. Create table
        sig_subPaths <- dplyr::count(table_data, model) %>%
            dplyr::arrange((model)) %>%
            knitr::kable(
                col.names = c("Model Type", "Count"),
                caption = "Sigificant Pathways by Model"
            ) %>%
            kableExtra::kable_paper(full_width = FALSE, html_font = "Cambria")

        # Return table
        return(sig_subPaths)
    }

    if (inherits(subpath_results, "list")) {
        # 2. Structure levels
        if (sum(grepl("interaction", names(subpath_results[[1]]))) == 0) {
            levs <- c("Single", "None")
        }
        if (!sum(grepl("interaction", names(subpath_results[[1]]))) == 0) {
            levs <- c("Interaction", "Parallel", "Single", "None")
        }

        for (i in seq_len(length(names(subpath_results)))) {
            # 3. Create table data for strata
            table_data <- subpath_results[[i]] %>%
                dplyr::mutate(model = factor(model, levels = levs)) %>%
                dplyr::select(sub_pathway, ends_with("_fisher"), model) %>%
                dplyr::distinct()

            if (i == 1) {
                # 4. Create table
                sig_subPaths <- dplyr::count(table_data, model) %>%
                    dplyr::arrange((model))
            } else {
                # Join strata results
                sig_subPaths <- sig_subPaths %>%
                    dplyr::left_join(dplyr::count(table_data, model) %>%
                        dplyr::arrange((model)), by = "model")
            }
        }

        # Modify table
        sig_subPaths <- sig_subPaths %>%
            knitr::kable(
                col.names = c("Model Type", names(subpath_results)),
                caption = "Sigificant Pathways by Model"
            ) %>%
            kableExtra::kable_paper(full_width = FALSE, html_font = "Cambria")
        # Return table
        return(sig_subPaths)
    }
}
