#' Proportion of the Significant Subpathways Within Superpathways
#'
#' Create a table that gives the percentage of significant subpathways within
#'  each superpathway.
#'
#' @param subpath_results Results data frame generated by
#'  \code{\link{subpathway_analysis}}
#'
#' @returns A table with the proportion (and percent) of significant subpathways
#'  within superpathways.
#'
#' @examples
#'
#' # Load data
#' data("demoDataSmall", package = "MetabolomicsPipeline")
#' dat <- demoDataSmall
#'
#' # Runsubpathay analysis
#' sub_analysis <- subpathway_analysis(dat,
#'     treat_var = "GROUP_NAME",
#'     block_var = "TIME1",
#'     strat_var = NULL,
#'     Assay = "normalized"
#' )
#'
#' #############################################################################
#' ### Results Plots ###########################################################
#' #############################################################################
#'
#' # significant subpathways by model type
#' subpath_by_model(sub_analysis)
#'
#' # Percentage of signficant subpathways within superpathways
#' subpath_within_superpath(sub_analysis)
#'
#' met_within_sub(sub_analysis, subpathway = "Aminosugar Metabolism")
#'
#' @importFrom dplyr case_when
#' @importFrom kableExtra kable_paper
#' @importFrom dplyr  summarise
#' @importFrom dplyr  n
#'
#'
#' @export
#'


subpath_within_superpath <- function(subpath_results) {
    if (inherits(subpath_results, "data.frame")) {
        # 2. Structure levels
        if (sum(grepl("interaction", names(subpath_results))) == 0) {
            levs <- c("Single", "None")

            cases <- expression(dplyr::case_when(
                model == "Single" ~ single_fisher
            ))
        }
        if (!sum(grepl("interaction", names(subpath_results))) == 0) {
            levs <- c("Interaction", "Parallel", "Single", "None")

            cases <- expression(dplyr::case_when(
                model == "Interaction" ~ interaction_fisher,
                model == "Parallel" ~ parallel_fisher,
                model == "Single" ~ single_fisher
            ))
        }


        # 3. Create table data
        table_data <- subpath_results %>%
            dplyr::mutate(model = factor(model, levels = levs)) %>%
            dplyr::select(
                sub_pathway, super_pathway,
                ends_with("_fisher"), model
            ) %>%
            dplyr::distinct()


        # 4. Formulate the Superpathway results table.
        superPath <- table_data %>%
            dplyr::group_by(super_pathway) %>%
            summarise(
                subpaths = n(),
                sig_paths = sum(model != "None"),
                percent_sig = mean(model != "None") * 100
            ) %>%
            dplyr::filter(!is.na(super_pathway)) %>%
            dplyr::arrange(-percent_sig) %>%
            dplyr::transmute(super_pathway,
                percent_significant = paste0(
                    sig_paths,
                    " / ", subpaths,
                    " (", round(percent_sig, 2), "%)"
                )
            ) %>%
            knitr::kable(
                col.names = c("Super Pathway", "Percent Significant"),
                caption = 
                  "Proportion of significant subpathways within superpathways"
            ) %>%
            kableExtra::kable_paper(full_width = FALSE, html_font = "Cambria")



        # Return table
        return(superPath)
    }

    if (inherits(subpath_results, "list")) {
        for (i in seq_len(length(subpath_results))) {
            # 2. Structure levels
            if (sum(grepl("interaction", names(subpath_results[[i]]))) == 0) {
                levs <- c("Single", "None")

                cases <- expression(dplyr::case_when(
                    model == "Single" ~ single_fisher
                ))
            }
            if (!sum(grepl("interaction", names(subpath_results[[i]]))) == 0) {
                levs <- c("Interaction", "Parallel", "Single", "None")

                cases <- expression(dplyr::case_when(
                    model == "Interaction" ~ interaction_fisher,
                    model == "Parallel" ~ parallel_fisher,
                    model == "Single" ~ single_fisher
                ))
            }


            table_data <- subpath_results[[i]] %>%
                dplyr::mutate(model = factor(model, levels = levs)) %>%
                dplyr::select(
                    sub_pathway, super_pathway,
                    ends_with("_fisher"), model
                ) %>%
                dplyr::distinct()


            if (i == 1) {
                # 4. Formulate the Superpathway results table.
                superPath <- table_data %>%
                    dplyr::group_by(super_pathway) %>%
                    dplyr::summarise(
                        subpaths = n(),
                        sig_paths = sum(model != "None", na.rm = TRUE),
                        percent_sig = mean(model != "None", na.rm = TRUE) * 100
                    ) %>%
                    dplyr::filter(!is.na(super_pathway)) %>%
                    dplyr::arrange(-percent_sig) %>%
                    dplyr::transmute(super_pathway,
                        percent_significant =
                            paste0(
                                sig_paths, " / ", subpaths,
                                " (", round(percent_sig, 2), "%)"
                            )
                    )
            } else {
                strat <- table_data %>%
                    dplyr::group_by(super_pathway) %>%
                    dplyr::summarise(
                        subpaths = n(),
                        sig_paths = sum(model != "None"),
                        percent_sig = mean(model != "None") * 100
                    ) %>%
                    dplyr::filter(!is.na(super_pathway)) %>%
                    dplyr::arrange(-percent_sig) %>%
                    dplyr::transmute(super_pathway,
                        percent_significant = paste0(
                            sig_paths, " / ",
                            subpaths,
                            " (", round(percent_sig, 2), "%)"
                        )
                    )

                superPath <- superPath %>%
                    left_join(strat, by = "super_pathway")
            }
        }
        superPath2 <- superPath %>%
            knitr::kable(
                col.names = c(
                    "Super Pathway",
                    paste0(
                        "Percent Significant \n (",
                        names(subpath_results), ")"
                    )
                ),
                caption =
                  "Proportion of significant subpathways within superpathways"
            ) %>%
            kableExtra::kable_paper(full_width = FALSE, html_font = "Cambria")

        return(superPath2)
    }
}
